#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import json
import re
from typing import TYPE_CHECKING, Any

import aiohttp

from airflow.providers.common.compat.connection import get_async_connection
from airflow.providers.http.hooks.http import HttpAsyncHook, HttpHook

if TYPE_CHECKING:
    from airflow.providers.common.compat.sdk import Connection


class DiscordCommonHandler:
    """Contains the common functionality."""

    def get_webhook_endpoint(self, conn: Connection | None, webhook_endpoint: str | None) -> str:
        """
        Return the default webhook endpoint or override if a webhook_endpoint is manually supplied.

        :param conn: Airflow Discord connection
        :param webhook_endpoint: The manually provided webhook endpoint
        :return: Webhook endpoint (str) to use
        """
        if webhook_endpoint:
            endpoint = webhook_endpoint
        elif conn:
            extra = conn.extra_dejson
            endpoint = extra.get("webhook_endpoint", "")
        else:
            raise ValueError(
                "Cannot get webhook endpoint: No valid Discord webhook endpoint or http_conn_id supplied."
            )

        # make sure endpoint matches the expected Discord webhook format
        if not re.fullmatch("webhooks/[0-9]+/[a-zA-Z0-9_-]+", endpoint):
            raise ValueError(
                'Expected Discord webhook endpoint in the form of "webhooks/{webhook.id}/{webhook.token}".'
            )

        return endpoint

    def build_discord_payload(
        self, *, tts: bool, message: str, username: str | None, avatar_url: str | None
    ) -> str:
        """
        Build a valid Discord JSON payload.

        :param tts: Is a text-to-speech message
        :param message: The message you want to send to your Discord channel
                        (max 2000 characters)
        :param username: Override the default username of the webhook
        :param avatar_url: Override the default avatar of the webhook
        :return: Discord payload (str) to send
        """
        if len(message) > 2000:
            raise ValueError("Discord message length must be 2000 or fewer characters.")
        payload: dict[str, Any] = {
            "content": message,
            "tts": tts,
        }
        if username:
            payload["username"] = username
        if avatar_url:
            payload["avatar_url"] = avatar_url
        return json.dumps(payload)


class DiscordWebhookHook(HttpHook):
    """
    This hook allows you to post messages to Discord using incoming webhooks.

    Takes a Discord connection ID with a default relative webhook endpoint. The
    default endpoint can be overridden using the webhook_endpoint parameter
    (https://discordapp.com/developers/docs/resources/webhook).

    Each Discord webhook can be pre-configured to use a specific username and
    avatar_url. You can override these defaults in this hook.

    :param http_conn_id: Http connection ID with host as "https://discord.com/api/" and
                         default webhook endpoint in the extra field in the form of
                         {"webhook_endpoint": "webhooks/{webhook.id}/{webhook.token}"}
    :param webhook_endpoint: Discord webhook endpoint in the form of
                             "webhooks/{webhook.id}/{webhook.token}"
    :param message: The message you want to send to your Discord channel
                    (max 2000 characters)
    :param username: Override the default username of the webhook
    :param avatar_url: Override the default avatar of the webhook
    :param tts: Is a text-to-speech message
    :param proxy: Proxy to use to make the Discord webhook call
    """

    conn_name_attr = "http_conn_id"
    default_conn_name = "discord_default"
    conn_type = "discord"
    hook_name = "Discord"

    @classmethod
    def get_connection_form_widgets(cls) -> dict[str, Any]:
        """Return connection widgets to add to Discord connection form."""
        from flask_appbuilder.fieldwidgets import BS3TextFieldWidget
        from flask_babel import lazy_gettext
        from wtforms import StringField
        from wtforms.validators import Optional

        return {
            "webhook_endpoint": StringField(
                lazy_gettext("Webhook Endpoint"),
                widget=BS3TextFieldWidget(),
                validators=[Optional()],
                default=None,
            ),
        }

    def __init__(
        self,
        http_conn_id: str | None = None,
        webhook_endpoint: str | None = None,
        message: str = "",
        username: str | None = None,
        avatar_url: str | None = None,
        tts: bool = False,
        proxy: str | None = None,
        *args: Any,
        **kwargs: Any,
    ) -> None:
        super().__init__(*args, **kwargs)
        self.handler = DiscordCommonHandler()
        self.http_conn_id: Any = http_conn_id
        self.webhook_endpoint = self._get_webhook_endpoint(http_conn_id, webhook_endpoint)
        self.message = message
        self.username = username
        self.avatar_url = avatar_url
        self.tts = tts
        self.proxy = proxy

    def _get_webhook_endpoint(self, http_conn_id: str | None, webhook_endpoint: str | None) -> str:
        """
        Return the default webhook endpoint or override if a webhook_endpoint is manually supplied.

        :param http_conn_id: The provided connection ID
        :param webhook_endpoint: The manually provided webhook endpoint
        :return: Webhook endpoint (str) to use
        """
        conn = None
        if not webhook_endpoint and http_conn_id:
            conn = self.get_connection(http_conn_id)
        return self.handler.get_webhook_endpoint(conn, webhook_endpoint)

    def execute(self) -> None:
        """Execute the Discord webhook call."""
        proxies = {}
        if self.proxy:
            # we only need https proxy for Discord
            proxies = {"https": self.proxy}

        discord_payload = self.handler.build_discord_payload(
            tts=self.tts, message=self.message, username=self.username, avatar_url=self.avatar_url
        )

        self.run(
            endpoint=self.webhook_endpoint,
            data=discord_payload,
            headers={"Content-type": "application/json"},
            extra_options={"proxies": proxies},
        )


class DiscordWebhookAsyncHook(HttpAsyncHook):
    """
    This hook allows you to post messages to Discord using incoming webhooks using async HTTP.

    Takes a Discord connection ID with a default relative webhook endpoint. The
    default endpoint can be overridden using the webhook_endpoint parameter
    (https://discordapp.com/developers/docs/resources/webhook).

    Each Discord webhook can be pre-configured to use a specific username and
    avatar_url. You can override these defaults in this hook.

    :param http_conn_id: Http connection ID with host as "https://discord.com/api/" and
                         default webhook endpoint in the extra field in the form of
                         {"webhook_endpoint": "webhooks/{webhook.id}/{webhook.token}"}
    :param webhook_endpoint: Discord webhook endpoint in the form of
                             "webhooks/{webhook.id}/{webhook.token}"
    :param message: The message you want to send to your Discord channel
                    (max 2000 characters)
    :param username: Override the default username of the webhook
    :param avatar_url: Override the default avatar of the webhook
    :param tts: Is a text-to-speech message
    :param proxy: Proxy to use to make the Discord webhook call
    """

    default_headers = {
        "Content-Type": "application/json",
    }
    conn_name_attr = "http_conn_id"
    default_conn_name = "discord_default"
    conn_type = "discord"
    hook_name = "Async Discord"

    def __init__(
        self,
        *,
        http_conn_id: str = "",
        webhook_endpoint: str | None = None,
        message: str = "",
        username: str | None = None,
        avatar_url: str | None = None,
        tts: bool = False,
        proxy: str | None = None,
        **kwargs: Any,
    ) -> None:
        super().__init__(**kwargs)
        self.http_conn_id = http_conn_id
        self.webhook_endpoint = webhook_endpoint
        self.message = message
        self.username = username
        self.avatar_url = avatar_url
        self.tts = tts
        self.proxy = proxy
        self.handler = DiscordCommonHandler()

    async def _get_webhook_endpoint(self) -> str:
        """
        Return the default webhook endpoint or override if a webhook_endpoint is manually supplied.

        :param http_conn_id: The provided connection ID
        :param webhook_endpoint: The manually provided webhook endpoint
        :return: Webhook endpoint (str) to use
        """
        conn = None
        if not self.webhook_endpoint and self.http_conn_id:
            conn = await get_async_connection(self.http_conn_id)
        return self.handler.get_webhook_endpoint(conn, self.webhook_endpoint)

    async def execute(self) -> None:
        """Execute the Discord webhook call."""
        webhook_endpoint = await self._get_webhook_endpoint()
        discord_payload = self.handler.build_discord_payload(
            tts=self.tts, message=self.message, username=self.username, avatar_url=self.avatar_url
        )

        async with aiohttp.ClientSession(proxy=self.proxy) as session:
            await super().run(
                session=session,
                endpoint=webhook_endpoint,
                data=discord_payload,
                headers=self.default_headers,
            )



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import json
from unittest import mock

import pytest
from aioresponses import aioresponses

from airflow.models import Connection
from airflow.providers.discord.hooks.discord_webhook import (
    DiscordCommonHandler,
    DiscordWebhookAsyncHook,
    DiscordWebhookHook,
)


@pytest.fixture
def aioresponse():
    """
    Creates mock async API response.
    """
    with aioresponses() as async_response:
        yield async_response


class TestDiscordCommonHandler:
    _config = {
        "message": "your message here",
        "username": "Airflow Webhook",
        "avatar_url": "https://static-cdn.avatars.com/my-avatar-path",
        "tts": False,
    }

    expected_payload_dict = {
        "content": _config["message"],
        "tts": _config["tts"],
        "username": _config["username"],
        "avatar_url": _config["avatar_url"],
    }

    expected_payload = json.dumps(expected_payload_dict)

    def test_get_webhook_endpoint_manual_token(self):
        provided_endpoint = "webhooks/11111/some-discord-token_111"
        handler = DiscordCommonHandler()
        webhook_endpoint = handler.get_webhook_endpoint(None, provided_endpoint)
        assert webhook_endpoint == provided_endpoint

    def test_get_webhook_endpoint_invalid_url(self):
        provided_endpoint = "https://discordapp.com/some-invalid-webhook-url"
        handler = DiscordCommonHandler()
        expected_message = "Expected Discord webhook endpoint in the form of"
        with pytest.raises(ValueError, match=expected_message):
            handler.get_webhook_endpoint(None, provided_endpoint)

    def test_get_webhook_endpoint_conn_id(self):
        conn = Connection(
            conn_id="default-discord-webhook",
            conn_type="discord",
            host="https://discordapp.com/api/",
            extra='{"webhook_endpoint": "webhooks/00000/some-discord-token_000"}',
        )
        expected_webhook_endpoint = "webhooks/00000/some-discord-token_000"
        handler = DiscordCommonHandler()
        webhook_endpoint = handler.get_webhook_endpoint(conn, None)
        assert webhook_endpoint == expected_webhook_endpoint

    def test_build_discord_payload(self):
        handler = DiscordCommonHandler()
        payload = handler.build_discord_payload(**self._config)
        assert self.expected_payload == payload

    def test_build_discord_payload_message_length(self):
        # Given
        config = self._config.copy()
        # create message over the character limit
        config["message"] = "c" * 2001
        handler = DiscordCommonHandler()
        expected_message = "Discord message length must be 2000 or fewer characters"
        with pytest.raises(ValueError, match=expected_message):
            handler.build_discord_payload(**config)


class TestDiscordWebhookHook:
    @pytest.fixture(autouse=True)
    def setup_connections(self, create_connection_without_db):
        create_connection_without_db(
            Connection(
                conn_id="default-discord-webhook",
                conn_type="discord",
                host="https://discordapp.com/api/",
                extra='{"webhook_endpoint": "webhooks/00000/some-discord-token_000"}',
            )
        )

    def test_get_webhook_endpoint_manual_token(self):
        # Given
        provided_endpoint = "webhooks/11111/some-discord-token_111"
        hook = DiscordWebhookHook(webhook_endpoint=provided_endpoint)

        # When
        webhook_endpoint = hook._get_webhook_endpoint(None, provided_endpoint)

        # Then
        assert webhook_endpoint == provided_endpoint

    def test_get_webhook_endpoint_conn_id(self):
        # Given
        conn_id = "default-discord-webhook"
        hook = DiscordWebhookHook(http_conn_id=conn_id)
        expected_webhook_endpoint = "webhooks/00000/some-discord-token_000"

        # When
        webhook_endpoint = hook._get_webhook_endpoint(conn_id, None)

        # Then
        assert webhook_endpoint == expected_webhook_endpoint


class TestDiscordWebhookAsyncHook:
    @pytest.fixture(autouse=True)
    def setup_connections(self, create_connection_without_db):
        create_connection_without_db(
            Connection(
                conn_id="default-discord-webhook",
                conn_type="discord",
                host="https://discordapp.com/api/",
                extra='{"webhook_endpoint": "webhooks/00000/some-discord-token_000"}',
            )
        )

    @pytest.fixture(autouse=True)
    def mock_get_connection(self):
        """Mock the async connection retrieval."""
        with mock.patch(
            "airflow.providers.discord.hooks.discord_webhook.get_async_connection",
            new_callable=mock.AsyncMock,
        ) as mock_conn:
            mock_conn.return_value = Connection(
                conn_id="default-discord-webhook",
                conn_type="discord",
                host="https://discordapp.com/api/",
                extra='{"webhook_endpoint": "webhooks/00000/some-discord-token_000"}',
            )
            yield mock_conn

    @pytest.mark.asyncio
    async def test_manual_token_overrides_conn(self):
        provided_endpoint = "webhooks/11111/some-discord-token_111"
        hook = DiscordWebhookAsyncHook(webhook_endpoint=provided_endpoint)
        webhook_endpoint = await hook._get_webhook_endpoint()
        assert webhook_endpoint == provided_endpoint

    @pytest.mark.asyncio
    async def test_get_webhook_endpoint_conn_id(self):
        conn_id = "default-discord-webhook"
        hook = DiscordWebhookAsyncHook(http_conn_id=conn_id)
        expected_webhook_endpoint = "webhooks/00000/some-discord-token_000"
        webhook_endpoint = await hook._get_webhook_endpoint()
        assert webhook_endpoint == expected_webhook_endpoint

    @pytest.mark.asyncio
    async def test_execute_with_payload(self):
        conn_id = "default-discord-webhook"
        hook = DiscordWebhookAsyncHook(
            http_conn_id=conn_id,
            message="your message here",
            username="Airflow Webhook",
            avatar_url="https://static-cdn.avatars.com/my-avatar-path",
            tts=False,
        )
        expected_payload_dict = {
            "content": "your message here",
            "tts": False,
            "username": "Airflow Webhook",
            "avatar_url": "https://static-cdn.avatars.com/my-avatar-path",
        }

        with mock.patch("aiohttp.ClientSession.post", new_callable=mock.AsyncMock) as mocked_function:
            await hook.execute()
            assert mocked_function.call_args.kwargs.get("data") == json.dumps(expected_payload_dict)

    @pytest.mark.asyncio
    async def test_execute_with_success(self, aioresponse):
        conn_id = "default-discord-webhook"
        hook = DiscordWebhookAsyncHook(
            http_conn_id=conn_id,
            message="your message here",
            username="Airflow Webhook",
            avatar_url="https://static-cdn.avatars.com/my-avatar-path",
            tts=False,
        )
        aioresponse.post("https://discordapp.com/api/webhooks/00000/some-discord-token_000", status=200)
        await hook.execute()



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest.mock import AsyncMock, MagicMock, call, patch

import pytest

from airflow.models import Connection
from airflow.providers.discord.notifications.discord import DiscordNotifier


@pytest.fixture(autouse=True)
def setup_connections(create_connection_without_db):
    create_connection_without_db(
        Connection(
            conn_id="my_discord_conn_id",
            conn_type="discord",
            host="https://discordapp.com/api/",
            extra='{"webhook_endpoint": "webhooks/00000/some-discord-token_000"}',
        )
    )


@patch("airflow.providers.discord.notifications.discord.DiscordWebhookHook.execute")
def test_discord_notifier_notify(mock_execute):
    notifier = DiscordNotifier(
        discord_conn_id="my_discord_conn_id",
        text="This is a test message",
        username="test_user",
        avatar_url="https://example.com/avatar.png",
        tts=False,
    )
    context = MagicMock()

    notifier.notify(context)

    mock_execute.assert_called_once()
    assert notifier.hook.username == "test_user"
    assert notifier.hook.message == "This is a test message"
    assert notifier.hook.avatar_url == "https://example.com/avatar.png"
    assert notifier.hook.tts is False


@pytest.mark.asyncio
@patch(
    "airflow.providers.discord.notifications.discord.DiscordWebhookAsyncHook.execute",
    new_callable=AsyncMock,
)
async def test_async_notifier(mock_async_hook):
    notifier = DiscordNotifier(
        discord_conn_id="my_discord_conn_id",
        text="This is a test message",
        username="test_user",
        avatar_url="https://example.com/avatar.png",
        tts=False,
    )
    await notifier.async_notify({})
    assert mock_async_hook.mock_calls == [call()]



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from airflow.models.dag import DAG
from airflow.providers.discord.operators.discord_webhook import DiscordWebhookOperator
from airflow.utils import timezone

DEFAULT_DATE = timezone.datetime(2018, 1, 1)


class TestDiscordWebhookOperator:
    _config = {
        "http_conn_id": "discord-webhook-default",
        "webhook_endpoint": "webhooks/11111/some-discord-token_111",
        "message": "your message here",
        "username": "Airflow Webhook",
        "avatar_url": "https://static-cdn.avatars.com/my-avatar-path",
        "tts": False,
        "proxy": "https://proxy.proxy.com:8888",
    }

    def setup_method(self):
        args = {"owner": "airflow", "start_date": DEFAULT_DATE}
        self.dag = DAG("test_dag_id", schedule=None, default_args=args)

    def test_execute(self):
        operator = DiscordWebhookOperator(task_id="discord_webhook_task", dag=self.dag, **self._config)

        assert self._config["http_conn_id"] == operator.http_conn_id
        assert self._config["webhook_endpoint"] == operator.webhook_endpoint
        assert self._config["message"] == operator.message
        assert self._config["username"] == operator.username
        assert self._config["avatar_url"] == operator.avatar_url
        assert self._config["tts"] == operator.tts
        assert self._config["proxy"] == operator.proxy



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from collections.abc import Sequence
from typing import TYPE_CHECKING

from airflow.exceptions import AirflowException
from airflow.providers.discord.hooks.discord_webhook import DiscordWebhookHook
from airflow.providers.http.operators.http import HttpOperator

if TYPE_CHECKING:
    from airflow.providers.common.compat.sdk import Context


class DiscordWebhookOperator(HttpOperator):
    """
    This operator allows you to post messages to Discord using incoming webhooks.

    Takes a Discord connection ID with a default relative webhook endpoint. The
    default endpoint can be overridden using the webhook_endpoint parameter
    (https://discordapp.com/developers/docs/resources/webhook).

    Each Discord webhook can be pre-configured to use a specific username and
    avatar_url. You can override these defaults in this operator.

    :param http_conn_id: Http connection ID with host as "https://discord.com/api/" and
                         default webhook endpoint in the extra field in the form of
                         {"webhook_endpoint": "webhooks/{webhook.id}/{webhook.token}"}
    :param webhook_endpoint: Discord webhook endpoint in the form of
                             "webhooks/{webhook.id}/{webhook.token}" (templated)
    :param message: The message you want to send to your Discord channel
                    (max 2000 characters). (templated)
    :param username: Override the default username of the webhook. (templated)
    :param avatar_url: Override the default avatar of the webhook
    :param tts: Is a text-to-speech message
    :param proxy: Proxy to use to make the Discord webhook call
    """

    template_fields: Sequence[str] = ("username", "message", "webhook_endpoint")

    def __init__(
        self,
        *,
        http_conn_id: str | None = None,
        webhook_endpoint: str | None = None,
        message: str = "",
        username: str | None = None,
        avatar_url: str | None = None,
        tts: bool = False,
        proxy: str | None = None,
        **kwargs,
    ) -> None:
        super().__init__(endpoint=webhook_endpoint, **kwargs)

        if not http_conn_id:
            raise AirflowException("No valid Discord http_conn_id supplied.")

        self.http_conn_id = http_conn_id
        self.webhook_endpoint = webhook_endpoint
        self.message = message
        self.username = username
        self.avatar_url = avatar_url
        self.tts = tts
        self.proxy = proxy

    @property
    def hook(self) -> DiscordWebhookHook:
        hook = DiscordWebhookHook(
            self.http_conn_id,
            self.webhook_endpoint,
            self.message,
            self.username,
            self.avatar_url,
            self.tts,
            self.proxy,
        )
        return hook

    def execute(self, context: Context) -> None:
        """
        Call the DiscordWebhookHook to post a message.

        :param context: the context object
        :return: None
        """
        self.hook.execute()



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from functools import cached_property

from airflow.providers.common.compat.notifier import BaseNotifier
from airflow.providers.discord.hooks.discord_webhook import DiscordWebhookAsyncHook, DiscordWebhookHook
from airflow.providers.discord.version_compat import AIRFLOW_V_3_1_PLUS

ICON_URL: str = (
    "https://raw.githubusercontent.com/apache/airflow/main/airflow-core/src/airflow/ui/public/pin_100.png"
)


class DiscordNotifier(BaseNotifier):
    """
    Discord BaseNotifier.

    :param discord_conn_id: Http connection ID with host as "https://discord.com/api/" and
                         default webhook endpoint in the extra field in the form of
                         {"webhook_endpoint": "webhooks/{webhook.id}/{webhook.token}"}
    :param text: The content of the message
    :param username: The username to send the message as. Optional
    :param avatar_url: The URL of the avatar to use for the message. Optional
    :param tts: Text to speech.
    """

    # A property that specifies the attributes that can be templated.
    template_fields = ("discord_conn_id", "text", "username", "avatar_url", "tts")

    def __init__(
        self,
        discord_conn_id: str = "discord_webhook_default",
        text: str = "This is a default message",
        username: str = "Airflow",
        avatar_url: str = ICON_URL,
        tts: bool = False,
        **kwargs,
    ):
        if AIRFLOW_V_3_1_PLUS:
            #  Support for passing context was added in 3.1.0
            super().__init__(**kwargs)
        else:
            super().__init__()
        self.discord_conn_id = discord_conn_id
        self.text = text
        self.username = username
        self.avatar_url = avatar_url

        # If you're having problems with tts not being recognized in __init__(),
        # you can define that after instantiating the class
        self.tts = tts

    @cached_property
    def hook(self) -> DiscordWebhookHook:
        """Discord Webhook Hook."""
        return DiscordWebhookHook(http_conn_id=self.discord_conn_id)

    @cached_property
    def hook_async(self) -> DiscordWebhookAsyncHook:
        """Discord Webhook Async Hook."""
        return DiscordWebhookAsyncHook(
            http_conn_id=self.discord_conn_id,
            message=self.text,
            username=self.username,
            avatar_url=self.avatar_url,
            tts=self.tts,
        )

    def notify(self, context):
        """
        Send a message to a Discord channel.

        :param context: the context object
        :return: None
        """
        self.hook.username = self.username
        self.hook.message = self.text
        self.hook.avatar_url = self.avatar_url
        self.hook.tts = self.tts

        self.hook.execute()

    async def async_notify(self, context) -> None:
        """
        Send a message to a Discord channel using async HTTP.

        :param context: the context object
        :return: None
        """
        await self.hook_async.execute()
